import binascii
import hashlib

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QApplication,
    QDialog,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
)


class RegisterUser(QDialog):
    """Класс диалог регистрации пользователя на сервере."""

    def __init__(self, database, server):
        super().__init__()

        self.database = database
        self.server = server

        self.setWindowTitle("Регистрация")
        self.setFixedSize(175, 183)
        self.setModal(True)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.label_username = QLabel("Введите имя пользователя:", self)
        self.label_username.move(10, 10)
        self.label_username.setFixedSize(150, 15)

        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(154, 20)
        self.client_name.move(10, 30)

        self.label_passwd = QLabel("Введите пароль:", self)
        self.label_passwd.move(10, 55)
        self.label_passwd.setFixedSize(150, 15)

        self.client_passwd = QLineEdit(self)
        self.client_passwd.setFixedSize(154, 20)
        self.client_passwd.move(10, 75)
        self.client_passwd.setEchoMode(QLineEdit.Password)
        self.label_conf = QLabel("Введите подтверждение:", self)
        self.label_conf.move(10, 100)
        self.label_conf.setFixedSize(150, 15)

        self.client_conf = QLineEdit(self)
        self.client_conf.setFixedSize(154, 20)
        self.client_conf.move(10, 120)
        self.client_conf.setEchoMode(QLineEdit.Password)

        self.btn_ok = QPushButton("Сохранить", self)
        self.btn_ok.move(10, 150)
        self.btn_ok.clicked.connect(self.save_data)

        self.btn_cancel = QPushButton("Выход", self)
        self.btn_cancel.move(90, 150)
        self.btn_cancel.clicked.connect(self.close)

        self.messages = QMessageBox()

        self.show()

    def save_data(self):
        """
        Метод проверки правильности ввода и сохранения в базу нового пользователя.
        """
        if not self.client_name.text():
            self.messages.critical(
                self, "Ошибка", "Не указано имя пользователя."
            )
            return
        elif self.client_passwd.text() != self.client_conf.text():
            self.messages.critical(
                self, "Ошибка", "Введённые пароли не совпадают."
            )
            return
        elif self.database.check_user(self.client_name.text()):
            self.messages.critical(
                self, "Ошибка", "Пользователь уже существует."
            )
            return
        else:
            # Генерируем хэш пароля, в качестве соли будем использовать логин в
            # нижнем регистре.
            passwd_bytes = self.client_passwd.text().encode("utf-8")
            salt = self.client_name.text().lower().encode("utf-8")
            passwd_hash = hashlib.pbkdf2_hmac(
                "sha512", passwd_bytes, salt, 10000
            )
            self.database.add_user(
                self.client_name.text(), binascii.hexlify(passwd_hash)
            )
            self.messages.information(
                self, "Успех", "Пользователь успешно зарегистрирован."
            )
            # Рассылаем клиентам сообщение о необходимости обновить справочники
            self.server.service_update_lists()
            self.close()


if __name__ == "__main__":
    app = QApplication([])
    from database import ServerStorage

    database = ServerStorage("../server_database.db3")
    import os
    import sys

    path1 = os.path.join(os.getcwd(), "../../..")
    sys.path.insert(0, path1)
    from core import MessageProcessor

    server = MessageProcessor("127.0.0.1", 7777, database)
    dial = RegisterUser(database, server)
    app.exec_()



import sys

sys.path.append("../../../")
import logging
import logging.handlers
import os

from common.variables import LOGGING_LEVEL

# создаём формировщик логов (formatter):
server_formatter = logging.Formatter(
    "%(asctime)s %(levelname)s %(filename)s %(message)s"
)

# Подготовка имени файла для логирования
path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "server.log")

# создаём потоки вывода логов
steam = logging.StreamHandler(sys.stderr)
steam.setFormatter(server_formatter)
steam.setLevel(logging.INFO)
log_file = logging.handlers.TimedRotatingFileHandler(
    path, encoding="utf8", interval=1, when="D"
)
log_file.setFormatter(server_formatter)

# создаём регистратор и настраиваем его
logger = logging.getLogger("server_dist")
logger.addHandler(steam)
logger.addHandler(log_file)
logger.setLevel(LOGGING_LEVEL)

# отладка
if __name__ == "__main__":
    logger.critical("Test critical event")
    logger.error("Test error ivent")
    logger.debug("Test debug ivent")
    logger.info("Test info ivent")



import logging

# Порт по умолчанию для сетевого ваимодействия
DEFAULT_PORT = 7777
# IP адрес по умолчанию для подключения клиента
DEFAULT_IP_ADDRESS = "127.0.0.1"
# Максимальная очередь подключений
MAX_CONNECTIONS = 5
# Максимальная длина сообщения в байтах
MAX_PACKAGE_LENGTH = 10240
# Кодировка проекта
ENCODING = "utf-8"
# Текущий уровень логирования
LOGGING_LEVEL = logging.DEBUG
# База данных для хранения данных сервера:
SERVER_CONFIG = "server_dist+++.ini"

# Протокол JIM основные ключи:
ACTION = "action"
TIME = "time"
USER = "user"
ACCOUNT_NAME = "account_name"
SENDER = "from"
DESTINATION = "to"
DATA = "bin"
PUBLIC_KEY = "pubkey"

# Прочие ключи, используемые в протоколе
PRESENCE = "presence"
RESPONSE = "response"
ERROR = "error"
MESSAGE = "message"
MESSAGE_TEXT = "mess_text"
EXIT = "exit"
GET_CONTACTS = "get_contacts"
LIST_INFO = "data_list"
REMOVE_CONTACT = "remove"
ADD_CONTACT = "add"
USERS_REQUEST = "get_users"
PUBLIC_KEY_REQUEST = "pubkey_need"

# Словари - ответы:
# 200
RESPONSE_200 = {RESPONSE: 200}
# 202
RESPONSE_202 = {RESPONSE: 202, LIST_INFO: None}
# 400
RESPONSE_400 = {RESPONSE: 400, ERROR: None}
# 205
RESPONSE_205 = {RESPONSE: 205}

# 511
RESPONSE_511 = {RESPONSE: 511, DATA: None}



from setuptools import setup, find_packages

setup(name="py_my_first_mess_server",
      version="0.1.0",
      description="Messenger Sever",
      author="Max",
      author_email="gatts@mail.ru",
      packages=find_packages(),
      install_requires=["PyQt5", "sqlalchemy", "pycryptodome", "pycryptodomex"],
      scripts=["server/server_run"],
      )
